// -*- C++ -*-
/*!
 * @file  LedControl.cpp * @brief Armadillo LED control component * $Date$ 
 *
 * $Id$ 
 */
#include "LedControl.h"

// Module specification
// <rtc-template block="module_spec">
static const char* ledcontrol_spec[] =
  {
    "implementation_id", "LedControl",
    "type_name",         "LedControl",
    "description",       "Armadillo LED control component",
    "version",           "1.0",
    "vendor",            "Noriaki Ando, AIST",
    "category",          "example",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    ""
  };
// </rtc-template>

LedControl::LedControl(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_ledinIn("ledin", m_ledin),
    m_ledstatusOut("ledstatus", m_ledstatus)

    // </rtc-template>
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("ledin", m_ledinIn);

  // Set OutPort buffer
  registerOutPort("ledstatus", m_ledstatusOut);

  // Set service provider to Ports

  // Set service consumers to Ports

  // Set CORBA Service Ports

  // </rtc-template>

}

LedControl::~LedControl()
{
}


RTC::ReturnCode_t LedControl::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable

  // </rtc-template>
  if (m_led.open())
  {
    return RTC::RTC_OK;
  } else {
    return RTC::RTC_ERROR;
  }
}


RTC::ReturnCode_t LedControl::onFinalize()
{
  m_led.close();
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t LedControl::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t LedControl::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t LedControl::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t LedControl::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t LedControl::onExecute(RTC::UniqueId ec_id)
{
  
  if (m_ledinIn.isNew())
    {
      m_ledinIn.read();

      int data = (int)m_ledin.data;
      if (!(data == 0 || data == 1 || data == 2 ||
	    data == 10 || data == 11 || data == 12 ||
	    data == 20 || data == 21 || data == 22))
	{
	  return RTC::RTC_OK;
	}

      int green = (int)(data % 10);
      int red   = data / 10;

      all_off();


      if ( green == 1 )
	{
	  m_led.green_on();
	}
      else if ( green == 2 )
	{
	  m_led.green_blink_on();
	}
      if ( red == 1 )
	{
	  m_led.red_on();
	}
      else if ( red == 2 )
	{
	  m_led.red_blink_on();
	}
      
      
    }
  int red = m_led.get_red_status() ? 10 : 0 ;
  int green = m_led.get_green_status() ? 1 : 0 ;

  m_ledstatus.data = red + green;
  m_ledstatusOut.write();

  usleep(100000);


  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t LedControl::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t LedControl::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t LedControl::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t LedControl::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t LedControl::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

void LedControl::all_off()
{
  m_led.red_off();
  m_led.green_off();
  m_led.green_blink_off();
  m_led.red_blink_off();
}

extern "C"
{
 
  void LedControlInit(RTC::Manager* manager)
  {
    RTC::Properties profile(ledcontrol_spec);
    manager->registerFactory(profile,
                             RTC::Create<LedControl>,
                             RTC::Delete<LedControl>);
  }
  
};



